#!/usr/bin/lua5.3
-- Authors:
--   - Jesse Laprade <jesselaprade@gmail.com>
--   - Will Sinatra <wpsinatra@gmail.com>
-- License: AGPLv3
local lume = nil
package.preload["lume"] = package.preload["lume"] or function(...)
  --
  -- lume
  --
  -- Copyright (c) 2020 rxi
  --
  -- Permission is hereby granted, free of charge, to any person obtaining a copy of
  -- this software and associated documentation files (the "Software"), to deal in
  -- the Software without restriction, including without limitation the rights to
  -- use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  -- of the Software, and to permit persons to whom the Software is furnished to do
  -- so, subject to the following conditions:
  --
  -- The above copyright notice and this permission notice shall be included in all
  -- copies or substantial portions of the Software.
  --
  -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  -- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  -- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  -- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  -- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  -- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  -- SOFTWARE.
  --
  
  local lume = { _version = "2.3.0" }
  
  local pairs, ipairs = pairs, ipairs
  local type, assert, unpack = type, assert, unpack or table.unpack
  local tostring, tonumber = tostring, tonumber
  local math_floor = math.floor
  local math_ceil = math.ceil
  local math_atan2 = math.atan2 or math.atan
  local math_sqrt = math.sqrt
  local math_abs = math.abs
  
  local noop = function()
  end
  
  local identity = function(x)
    return x
  end
  
  local patternescape = function(str)
    return str:gsub("[%(%)%.%%%+%-%*%?%[%]%^%$]", "%%%1")
  end
  
  local absindex = function(len, i)
    return i < 0 and (len + i + 1) or i
  end
  
  local iscallable = function(x)
    if type(x) == "function" then return true end
    local mt = getmetatable(x)
    return mt and mt.__call ~= nil
  end
  
  local getiter = function(x)
    if lume.isarray(x) then
      return ipairs
    elseif type(x) == "table" then
      return pairs
    end
    error("expected table", 3)
  end
  
  local iteratee = function(x)
    if x == nil then return identity end
    if iscallable(x) then return x end
    if type(x) == "table" then
      return function(z)
        for k, v in pairs(x) do
          if z[k] ~= v then return false end
        end
        return true
      end
    end
    return function(z) return z[x] end
  end
  
  
  
  function lume.clamp(x, min, max)
    return x < min and min or (x > max and max or x)
  end
  
  
  function lume.round(x, increment)
    if increment then return lume.round(x / increment) * increment end
    return x >= 0 and math_floor(x + .5) or math_ceil(x - .5)
  end
  
  
  function lume.sign(x)
    return x < 0 and -1 or 1
  end
  
  
  function lume.lerp(a, b, amount)
    return a + (b - a) * lume.clamp(amount, 0, 1)
  end
  
  
  function lume.smooth(a, b, amount)
    local t = lume.clamp(amount, 0, 1)
    local m = t * t * (3 - 2 * t)
    return a + (b - a) * m
  end
  
  
  function lume.pingpong(x)
    return 1 - math_abs(1 - x % 2)
  end
  
  
  function lume.distance(x1, y1, x2, y2, squared)
    local dx = x1 - x2
    local dy = y1 - y2
    local s = dx * dx + dy * dy
    return squared and s or math_sqrt(s)
  end
  
  
  function lume.angle(x1, y1, x2, y2)
    return math_atan2(y2 - y1, x2 - x1)
  end
  
  
  function lume.vector(angle, magnitude)
    return math.cos(angle) * magnitude, math.sin(angle) * magnitude
  end
  
  
  function lume.random(a, b)
    if not a then a, b = 0, 1 end
    if not b then b = 0 end
    return a + math.random() * (b - a)
  end
  
  
  function lume.randomchoice(t)
    return t[math.random(#t)]
  end
  
  
  function lume.weightedchoice(t)
    local sum = 0
    for _, v in pairs(t) do
      assert(v >= 0, "weight value less than zero")
      sum = sum + v
    end
    assert(sum ~= 0, "all weights are zero")
    local rnd = lume.random(sum)
    for k, v in pairs(t) do
      if rnd < v then return k end
      rnd = rnd - v
    end
  end
  
  
  function lume.isarray(x)
    return type(x) == "table" and x[1] ~= nil
  end
  
  
  function lume.push(t, ...)
    local n = select("#", ...)
    for i = 1, n do
      t[#t + 1] = select(i, ...)
    end
    return ...
  end
  
  
  function lume.remove(t, x)
    local iter = getiter(t)
    for i, v in iter(t) do
      if v == x then
        if lume.isarray(t) then
          table.remove(t, i)
          break
        else
          t[i] = nil
          break
        end
      end
    end
    return x
  end
  
  
  function lume.clear(t)
    local iter = getiter(t)
    for k in iter(t) do
      t[k] = nil
    end
    return t
  end
  
  
  function lume.extend(t, ...)
    for i = 1, select("#", ...) do
      local x = select(i, ...)
      if x then
        for k, v in pairs(x) do
          t[k] = v
        end
      end
    end
    return t
  end
  
  
  function lume.shuffle(t)
    local rtn = {}
    for i = 1, #t do
      local r = math.random(i)
      if r ~= i then
        rtn[i] = rtn[r]
      end
      rtn[r] = t[i]
    end
    return rtn
  end
  
  
  function lume.sort(t, comp)
    local rtn = lume.clone(t)
    if comp then
      if type(comp) == "string" then
        table.sort(rtn, function(a, b) return a[comp] < b[comp] end)
      else
        table.sort(rtn, comp)
      end
    else
      table.sort(rtn)
    end
    return rtn
  end
  
  
  function lume.array(...)
    local t = {}
    for x in ... do t[#t + 1] = x end
    return t
  end
  
  
  function lume.each(t, fn, ...)
    local iter = getiter(t)
    if type(fn) == "string" then
      for _, v in iter(t) do v[fn](v, ...) end
    else
      for _, v in iter(t) do fn(v, ...) end
    end
    return t
  end
  
  
  function lume.map(t, fn)
    fn = iteratee(fn)
    local iter = getiter(t)
    local rtn = {}
    for k, v in iter(t) do rtn[k] = fn(v) end
    return rtn
  end
  
  
  function lume.all(t, fn)
    fn = iteratee(fn)
    local iter = getiter(t)
    for _, v in iter(t) do
      if not fn(v) then return false end
    end
    return true
  end
  
  
  function lume.any(t, fn)
    fn = iteratee(fn)
    local iter = getiter(t)
    for _, v in iter(t) do
      if fn(v) then return true end
    end
    return false
  end
  
  
  function lume.reduce(t, fn, first)
    local started = first ~= nil
    local acc = first
    local iter = getiter(t)
    for _, v in iter(t) do
      if started then
        acc = fn(acc, v)
      else
        acc = v
        started = true
      end
    end
    assert(started, "reduce of an empty table with no first value")
    return acc
  end
  
  
  function lume.unique(t)
    local rtn = {}
    for k in pairs(lume.invert(t)) do
      rtn[#rtn + 1] = k
    end
    return rtn
  end
  
  
  function lume.filter(t, fn, retainkeys)
    fn = iteratee(fn)
    local iter = getiter(t)
    local rtn = {}
    if retainkeys then
      for k, v in iter(t) do
        if fn(v) then rtn[k] = v end
      end
    else
      for _, v in iter(t) do
        if fn(v) then rtn[#rtn + 1] = v end
      end
    end
    return rtn
  end
  
  
  function lume.reject(t, fn, retainkeys)
    fn = iteratee(fn)
    local iter = getiter(t)
    local rtn = {}
    if retainkeys then
      for k, v in iter(t) do
        if not fn(v) then rtn[k] = v end
      end
    else
      for _, v in iter(t) do
        if not fn(v) then rtn[#rtn + 1] = v end
      end
    end
    return rtn
  end
  
  
  function lume.merge(...)
    local rtn = {}
    for i = 1, select("#", ...) do
      local t = select(i, ...)
      local iter = getiter(t)
      for k, v in iter(t) do
        rtn[k] = v
      end
    end
    return rtn
  end
  
  
  function lume.concat(...)
    local rtn = {}
    for i = 1, select("#", ...) do
      local t = select(i, ...)
      if t ~= nil then
        local iter = getiter(t)
        for _, v in iter(t) do
          rtn[#rtn + 1] = v
        end
      end
    end
    return rtn
  end
  
  
  function lume.find(t, value)
    local iter = getiter(t)
    for k, v in iter(t) do
      if v == value then return k end
    end
    return nil
  end
  
  
  function lume.match(t, fn)
    fn = iteratee(fn)
    local iter = getiter(t)
    for k, v in iter(t) do
      if fn(v) then return v, k end
    end
    return nil
  end
  
  
  function lume.count(t, fn)
    local count = 0
    local iter = getiter(t)
    if fn then
      fn = iteratee(fn)
      for _, v in iter(t) do
        if fn(v) then count = count + 1 end
      end
    else
      if lume.isarray(t) then
        return #t
      end
      for _ in iter(t) do count = count + 1 end
    end
    return count
  end
  
  
  function lume.slice(t, i, j)
    i = i and absindex(#t, i) or 1
    j = j and absindex(#t, j) or #t
    local rtn = {}
    for x = i < 1 and 1 or i, j > #t and #t or j do
      rtn[#rtn + 1] = t[x]
    end
    return rtn
  end
  
  
  function lume.first(t, n)
    if not n then return t[1] end
    return lume.slice(t, 1, n)
  end
  
  
  function lume.last(t, n)
    if not n then return t[#t] end
    return lume.slice(t, -n, -1)
  end
  
  
  function lume.invert(t)
    local rtn = {}
    for k, v in pairs(t) do rtn[v] = k end
    return rtn
  end
  
  
  function lume.pick(t, ...)
    local rtn = {}
    for i = 1, select("#", ...) do
      local k = select(i, ...)
      rtn[k] = t[k]
    end
    return rtn
  end
  
  
  function lume.keys(t)
    local rtn = {}
    local iter = getiter(t)
    for k in iter(t) do rtn[#rtn + 1] = k end
    return rtn
  end
  
  
  function lume.clone(t)
    local rtn = {}
    for k, v in pairs(t) do rtn[k] = v end
    return rtn
  end
  
  
  function lume.fn(fn, ...)
    assert(iscallable(fn), "expected a function as the first argument")
    local args = { ... }
    return function(...)
      local a = lume.concat(args, { ... })
      return fn(unpack(a))
    end
  end
  
  
  function lume.once(fn, ...)
    local f = lume.fn(fn, ...)
    local done = false
    return function(...)
      if done then return end
      done = true
      return f(...)
    end
  end
  
  
  local memoize_fnkey = {}
  local memoize_nil = {}
  
  function lume.memoize(fn)
    local cache = {}
    return function(...)
      local c = cache
      for i = 1, select("#", ...) do
        local a = select(i, ...) or memoize_nil
        c[a] = c[a] or {}
        c = c[a]
      end
      c[memoize_fnkey] = c[memoize_fnkey] or {fn(...)}
      return unpack(c[memoize_fnkey])
    end
  end
  
  
  function lume.combine(...)
    local n = select('#', ...)
    if n == 0 then return noop end
    if n == 1 then
      local fn = select(1, ...)
      if not fn then return noop end
      assert(iscallable(fn), "expected a function or nil")
      return fn
    end
    local funcs = {}
    for i = 1, n do
      local fn = select(i, ...)
      if fn ~= nil then
        assert(iscallable(fn), "expected a function or nil")
        funcs[#funcs + 1] = fn
      end
    end
    return function(...)
      for _, f in ipairs(funcs) do f(...) end
    end
  end
  
  
  function lume.call(fn, ...)
    if fn then
      return fn(...)
    end
  end
  
  
  function lume.time(fn, ...)
    local start = os.clock()
    local rtn = {fn(...)}
    return (os.clock() - start), unpack(rtn)
  end
  
  
  local lambda_cache = {}
  
  function lume.lambda(str)
    if not lambda_cache[str] then
      local args, body = str:match([[^([%w,_ ]-)%->(.-)$]])
      assert(args and body, "bad string lambda")
      local s = "return function(" .. args .. ")\nreturn " .. body .. "\nend"
      lambda_cache[str] = lume.dostring(s)
    end
    return lambda_cache[str]
  end
  
  
  local serialize
  
  local serialize_map = {
    [ "boolean" ] = tostring,
    [ "nil"     ] = tostring,
    [ "string"  ] = function(v) return string.format("%q", v) end,
    [ "number"  ] = function(v)
      if      v ~=  v     then return  "0/0"      --  nan
      elseif  v ==  1 / 0 then return  "1/0"      --  inf
      elseif  v == -1 / 0 then return "-1/0" end  -- -inf
      return tostring(v)
    end,
    [ "table"   ] = function(t, stk)
      stk = stk or {}
      if stk[t] then error("circular reference") end
      local rtn = {}
      stk[t] = true
      for k, v in pairs(t) do
        rtn[#rtn + 1] = "[" .. serialize(k, stk) .. "]=" .. serialize(v, stk)
      end
      stk[t] = nil
      return "{" .. table.concat(rtn, ",") .. "}"
    end
  }
  
  setmetatable(serialize_map, {
    __index = function(_, k) error("unsupported serialize type: " .. k) end
  })
  
  serialize = function(x, stk)
    return serialize_map[type(x)](x, stk)
  end
  
  function lume.serialize(x)
    return serialize(x)
  end
  
  
  function lume.deserialize(str)
    return lume.dostring("return " .. str)
  end
  
  
  function lume.split(str, sep)
    if not sep then
      return lume.array(str:gmatch("([%S]+)"))
    else
      assert(sep ~= "", "empty separator")
      local psep = patternescape(sep)
      return lume.array((str..sep):gmatch("(.-)("..psep..")"))
    end
  end
  
  
  function lume.trim(str, chars)
    if not chars then return str:match("^[%s]*(.-)[%s]*$") end
    chars = patternescape(chars)
    return str:match("^[" .. chars .. "]*(.-)[" .. chars .. "]*$")
  end
  
  
  function lume.wordwrap(str, limit)
    limit = limit or 72
    local check
    if type(limit) == "number" then
      check = function(s) return #s >= limit end
    else
      check = limit
    end
    local rtn = {}
    local line = ""
    for word, spaces in str:gmatch("(%S+)(%s*)") do
      local s = line .. word
      if check(s) then
        table.insert(rtn, line .. "\n")
        line = word
      else
        line = s
      end
      for c in spaces:gmatch(".") do
        if c == "\n" then
          table.insert(rtn, line .. "\n")
          line = ""
        else
          line = line .. c
        end
      end
    end
    table.insert(rtn, line)
    return table.concat(rtn)
  end
  
  
  function lume.format(str, vars)
    if not vars then return str end
    local f = function(x)
      return tostring(vars[x] or vars[tonumber(x)] or "{" .. x .. "}")
    end
    return (str:gsub("{(.-)}", f))
  end
  
  
  function lume.trace(...)
    local info = debug.getinfo(2, "Sl")
    local t = { info.short_src .. ":" .. info.currentline .. ":" }
    for i = 1, select("#", ...) do
      local x = select(i, ...)
      if type(x) == "number" then
        x = string.format("%g", lume.round(x, .01))
      end
      t[#t + 1] = tostring(x)
    end
    print(table.concat(t, " "))
  end
  
  
  function lume.dostring(str)
    return assert((loadstring or load)(str))()
  end
  
  
  function lume.uuid()
    local fn = function(x)
      local r = math.random(16) - 1
      r = (x == "x") and (r + 1) or (r % 4) + 9
      return ("0123456789abcdef"):sub(r, r)
    end
    return (("xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx"):gsub("[xy]", fn))
  end
  
  
  function lume.hotswap(modname)
    local oldglobal = lume.clone(_G)
    local updated = {}
    local function update(old, new)
      if updated[old] then return end
      updated[old] = true
      local oldmt, newmt = getmetatable(old), getmetatable(new)
      if oldmt and newmt then update(oldmt, newmt) end
      for k, v in pairs(new) do
        if type(v) == "table" then update(old[k], v) else old[k] = v end
      end
    end
    local err = nil
    local function onerror(e)
      for k in pairs(_G) do _G[k] = oldglobal[k] end
      err = lume.trim(e)
    end
    local ok, oldmod = pcall(require, modname)
    oldmod = ok and oldmod or nil
    xpcall(function()
      package.loaded[modname] = nil
      local newmod = require(modname)
      if type(oldmod) == "table" then update(oldmod, newmod) end
      for k, v in pairs(oldglobal) do
        if v ~= _G[k] and type(v) == "table" then
          update(v, _G[k])
          _G[k] = v
        end
      end
    end, onerror)
    package.loaded[modname] = oldmod
    if err then return nil, err end
    return oldmod
  end
  
  
  local ripairs_iter = function(t, i)
    i = i - 1
    local v = t[i]
    if v ~= nil then
      return i, v
    end
  end
  
  function lume.ripairs(t)
    return ripairs_iter, t, (#t + 1)
  end
  
  
  function lume.color(str, mul)
    mul = mul or 1
    local r, g, b, a
    r, g, b = str:match("#(%x%x)(%x%x)(%x%x)")
    if r then
      r = tonumber(r, 16) / 0xff
      g = tonumber(g, 16) / 0xff
      b = tonumber(b, 16) / 0xff
      a = 1
    elseif str:match("rgba?%s*%([%d%s%.,]+%)") then
      local f = str:gmatch("[%d.]+")
      r = (f() or 0) / 0xff
      g = (f() or 0) / 0xff
      b = (f() or 0) / 0xff
      a = f() or 1
    else
      error(("bad color string '%s'"):format(str))
    end
    return r * mul, g * mul, b * mul, a * mul
  end
  
  
  local chain_mt = {}
  chain_mt.__index = lume.map(lume.filter(lume, iscallable, true),
    function(fn)
      return function(self, ...)
        self._value = fn(self._value, ...)
        return self
      end
    end)
  chain_mt.__index.result = function(x) return x._value end
  
  function lume.chain(value)
    return setmetatable({ _value = value }, chain_mt)
  end
  
  setmetatable(lume,  {
    __call = function(_, ...)
      return lume.chain(...)
    end
  })
  
  
  return lume
end
lume = require("lume")
local home = os.getenv("HOME")
local program_name = "fa"
local program_version = "0.5.0"
local program_file = (home .. "/." .. program_name)
local event_symbol = "!"
local keyword_today = "today"
local cmds_help = {human = "help", long = "--help", short = "-h"}
local cmds_version = {human = "version", long = "--version", short = "-v"}
local cmd_init = "init"
local cmd_add = "add"
local cmd_rm = "rm"
local cmd_ls = "ls"
local cmd_notify = "notify"
local newline = "\n"
local double_newline = "\n\n"
local space = " "
local indent_2 = "  "
local messages = {["agenda-empty"] = "There is nothing in your agenda.", ["cancelling-date-creation"] = "Cancelling date creation", ["date-already-exists"] = ("The following event already exists in your agenda:" .. newline .. "%s %s" .. newline .. "Choose one of the options below:" .. newline .. "1. Replace the old event" .. newline .. "2. Add the event to the existing date" .. newline .. "3. Cancell even creation" .. newline .. "> "), ["date-format"] = ("Error: Date format must be mmdd" .. newline .. "Example: 1231 \"The last day of december\""), ["file-already-exists"] = string.format("A file or directory named '%s' already exists.", program_file), ["file-doesnt-exist"] = string.format("%s doesn't exist. Try running '%s %s' (without the quotation marks).", program_file, program_name, cmd_init), ["key-doesnt-exist"] = "The date '%s' wasn't found in your agenda.", ["not-an-option"] = "Error: '%s' is not an option.", ["program-version"] = string.format("AGPLv3 -- %s", program_version), ["sub-key-doesnt-exist"] = "The item '%s' on %s wasn't found in your agenda.", added = "Added '%s' to your agenda.", initialized = string.format("Successfully created %s", program_file), removed = "Removed '%s' from your agenda.", usage = string.format("For usage, type '%s %s'.", program_name, cmds_help.human)}
local function key_exists_3f(tbl, key)
  if (type(tbl) == "table") then
    local tbl_keys = lume.keys(tbl)
    local function _0_(_241)
      return (key == _241)
    end
    if lume.any(tbl_keys, _0_) then
      return true
    else
      return false
    end
  else
    return false
  end
end
local function get_date()
  return os.date("%m%d")
end
local function file_exists_3f(str)
  local file_in = io.open(str, "r")
  if file_in then
    return file_in:close()
  else
    return false
  end
end
local function file__3etable(str)
  if file_exists_3f(str) then
    local file_in = io.open(str, "r")
    local function close_handlers_0_(ok_0_, ...)
      file_in:close()
      if ok_0_ then
        return ...
      else
        return error(..., 0)
      end
    end
    local function _0_()
      return lume.deserialize(file_in:read("*all"))
    end
    return close_handlers_0_(xpcall(_0_, (package.loaded.fennel or debug).traceback))
  else
    return nil
  end
end
local function table__3efile(str, tbl)
  if file_exists_3f(str) then
    local file_out = io.open(str, "w")
    local function close_handlers_0_(ok_0_, ...)
      file_out:close()
      if ok_0_ then
        return ...
      else
        return error(..., 0)
      end
    end
    local function _0_()
      return file_out:write(lume.serialize(tbl))
    end
    return close_handlers_0_(xpcall(_0_, (package.loaded.fennel or debug).traceback))
  else
    return nil
  end
end
local function create_prefix(seq, key)
  if (#seq > 1) then
    return string.format("%s. ", key)
  else
    return ""
  end
end
local function print_format(str, ...)
  return print(string.format(str, ...))
end
local function init_2fcreate_file()
  local file_out = io.open(program_file, "w")
  local function close_handlers_0_(ok_0_, ...)
    file_out:close()
    if ok_0_ then
      return ...
    else
      return error(..., 0)
    end
  end
  local function _0_()
    file_out:write(lume.serialize({}))
    return print(messages.initialized)
  end
  return close_handlers_0_(xpcall(_0_, (package.loaded.fennel or debug).traceback))
end
local function init()
  if file_exists_3f(program_file) then
    return print(messages["file-already-exists"])
  else
    return init_2fcreate_file()
  end
end
local function add_2fadd_event(date, event_str)
  local tbl = file__3etable(program_file)
  if key_exists_3f(tbl, date) then
    tbl[date][(1 + #tbl[date])] = event_str
  else
    tbl[date] = {event_str}
  end
  table__3efile(program_file, tbl)
  return print_format(messages.added, event_str)
end
local function add(date_str, event_str)
  if file_exists_3f(program_file) then
    local date = nil
    if (date_str == keyword_today) then
      date = get_date()
    else
      date = date_str
    end
    return add_2fadd_event(date, event_str)
  else
    return print(messages["file-doesnt-exist"])
  end
end
local function rm(date_str, event_str)
  if file_exists_3f(program_file) then
    local tbl = file__3etable(program_file)
    local date = nil
    if (date_str == keyword_today) then
      date = get_date()
    else
      date = date_str
    end
    local events_seq = tbl[date]
    local events_seq_index = tonumber(event_str)
    local date_exists_3f = key_exists_3f(tbl, date)
    local event_exists_3f = key_exists_3f(events_seq, events_seq_index)
    local _1_0 = {date_exists_3f, event_exists_3f, event_str}
    if ((type(_1_0) == "table") and (_1_0[1] == true) and (_1_0[2] == false) and (_1_0[3] == nil)) then
      tbl[date] = nil
      table__3efile(program_file, tbl)
      return print_format(messages.removed, date)
    elseif ((type(_1_0) == "table") and (_1_0[1] == true) and (_1_0[2] == true)) then
      local event = events_seq[events_seq_index]
      table.remove(events_seq, events_seq_index)
      if (#events_seq == 0) then
        tbl[date] = nil
        table__3efile(program_file, tbl)
        return print_format(messages.removed, event)
      else
        tbl[date] = events_seq
        table__3efile(program_file, tbl)
        return print_format(messages.removed, event)
      end
    elseif ((type(_1_0) == "table") and (_1_0[1] == true) and (_1_0[2] == false)) then
      return print_format(messages["sub-key-doesnt-exist"], event_str, date)
    elseif ((type(_1_0) == "table") and (_1_0[1] == false)) then
      return print_format(messages["key-doesnt-exist"], date)
    end
  else
    return print(messages["file-doesnt-exist"])
  end
end
local function notify()
  if file_exists_3f(program_file) then
    if key_exists_3f(file__3etable(program_file), get_date()) then
      return io.write(event_symbol)
    end
  else
    return print(messages["file-doesnt-exist"])
  end
end
local function ls_2fprint_seq(events_seq)
  local keys = lume.keys(events_seq)
  for _, key in pairs(keys) do
    local event = events_seq[key]
    local event_prefix = create_prefix(events_seq, key)
    print_format("%s%s", event_prefix, event)
  end
  return nil
end
local function ls_2fls_date(date_str)
  if file_exists_3f(program_file) then
    local tbl = file__3etable(program_file)
    local date = nil
    if (date_str == keyword_today) then
      date = get_date()
    else
      date = date_str
    end
    if key_exists_3f(tbl, date) then
      local events_seq = tbl[date]
      return ls_2fprint_seq(events_seq)
    end
  else
    return print(messages["file-doesnt-exist"])
  end
end
local function ls_2fprint_seq0(events_seq, date)
  local _0_ = events_seq
  local first_event = _0_[1]
  local _1_ = lume.keys(events_seq)
  local first_key = _1_[1]
  local rest_keys = {(table.unpack or unpack)(_1_, 2)}
  local first_event_prefix = create_prefix(events_seq, first_key)
  print_format("%s%s", first_event_prefix, first_event)
  for _, key in pairs(rest_keys) do
    local event = events_seq[key]
    local padding = string.rep(" ", (1 + #date))
    print_format("%s %s. %s", padding, key, event)
  end
  return nil
end
local function ls_2fsort_and_print_tbl(tbl, tbl_keys)
  table.sort(tbl_keys)
  for _, date in pairs(tbl_keys) do
    io.write(string.format("%s: ", date))
    local events_seq = tbl[date]
    ls_2fprint_seq0(events_seq, date)
  end
  return nil
end
local function ls(date_str)
  if file_exists_3f(program_file) then
    if date_str then
      return ls_2fls_date(date_str)
    else
      local tbl = file__3etable(program_file)
      local tbl_keys = lume.keys(tbl)
      local tbl_length = #tbl_keys
      if (tbl_length > 0) then
        return ls_2fsort_and_print_tbl(tbl, tbl_keys)
      else
        return print(messages["agenda-empty"])
      end
    end
  else
    return print(messages["file-doesnt-exist"])
  end
end
local function help()
  return print(("Usage:\n" .. "  fa <command> [<arg>] [<arg>]\n" .. "\n" .. "Commands:\n" .. "  init - Creates your agenda.\n" .. "  add <mmdd|today> \"Quoted text\" - Adds an event to your agenda.\n" .. "  rm <mmdd|today> - Removes a given date from your agenda.\n" .. "  rm <mmdd|today> [<number>] - Removes an event from given date from your agenda.\n" .. "  ls [<mmdd|today>] - Lists the events on the given date.\n" .. "  ls - Lists all dates and their events.\n" .. "  notify - Displays a \"!\" if an event exists today.\n" .. "  version - Prints the current version of fa.\n" .. "\n" .. "Examples:\n" .. "  fa init\n" .. "  fa add 1231 \"Sherry's birthday\"\n" .. "  fa add today \"Sherry's birthday\"\n" .. "  fa rm 1231\n" .. "  fa rm 1231 3 (See note below)\n" .. "  fa rm today\n" .. "  fa rm today 3 (See note below)\n" .. "  fa ls today\n" .. "  fa ls 1231\n" .. "  fa ls\n" .. "  fa notify\n" .. "  fa version\n" .. "\n" .. "Note: You may need to run 'fa ls' to see which number correlates to which event.\n"))
end
local function process_args(arg_tbl)
  local _0_0, _1_0, _2_0 = arg_tbl
  if ((type(_0_0) == "table") and (_0_0[1] == cmd_add) and (nil ~= _0_0[2]) and (nil ~= _0_0[3]) and (_0_0[4] == nil)) then
    local date_str = _0_0[2]
    local event_str = _0_0[3]
    return add(date_str, event_str)
  elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_rm) and (nil ~= _0_0[2]) and (nil ~= _0_0[3]) and (_0_0[4] == nil)) then
    local date_str = _0_0[2]
    local event_str = _0_0[3]
    return rm(date_str, event_str)
  elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_rm) and (nil ~= _0_0[2]) and (_0_0[3] == nil)) then
    local date_str = _0_0[2]
    return rm(date_str)
  elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_ls) and (nil ~= _0_0[2]) and (_0_0[3] == nil)) then
    local date_str = _0_0[2]
    return ls(date_str)
  elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_ls) and (_0_0[2] == nil)) then
    return ls()
  else
    local _3_
    do
      local cmd = _0_0[1]
      _3_ = (((type(_0_0) == "table") and (nil ~= _0_0[1]) and (_0_0[2] == nil)) and ((cmd == cmds_help.human) or (cmd == cmds_help.long) or (cmd == cmds_help.short)))
    end
    if _3_ then
      local cmd = _0_0[1]
      return help()
    else
      local _4_
      do
        local cmd = _0_0[1]
        _4_ = (((type(_0_0) == "table") and (nil ~= _0_0[1]) and (_0_0[2] == nil)) and ((cmd == cmds_version.human) or (cmd == cmds_version.long) or (cmd == cmds_version.short)))
      end
      if _4_ then
        local cmd = _0_0[1]
        return print(messages["program-version"])
      elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_init) and (_0_0[2] == nil)) then
        return init()
      elseif ((type(_0_0) == "table") and (_0_0[1] == cmd_notify) and (_0_0[2] == nil)) then
        return notify()
      else
        local _ = _0_0
        return print(messages.usage)
      end
    end
  end
end
local function main(arg_tbl)
  return process_args(arg_tbl)
end
return main(arg)
