#!/usr/bin/env fennel

;; For debugging ----------------------;;
;; (local view (require :fennelview))
;; (global pp (fn [x] (print (view x))))
;; ------------------------------------;;

(local lume (require :lume))
(local home (os.getenv "HOME"))

(local program-name "fa")
(local program-version "0.5.0")
(local program-file (.. home "/." program-name))

;; Not in use (yet?)
;; This gets lowercased by (get-day)
;; (local day-fmt "%a")

(local event-symbol "!")

(local keyword-today "today")

(local cmds-help
  {:human "help"
   :long  "--help"
   :short "-h"})
(local cmds-version
  {:human "version"
   :long  "--version"
   :short "-v"})
(local cmd-init "init")
(local cmd-add "add")
(local cmd-rm "rm")
(local cmd-ls "ls")
(local cmd-notify "notify")

(local newline "\n")
(local double-newline "\n\n")
(local space " ")
(local indent-2 "  ")

(local messages
  {:usage
   (string.format "For usage, type '%s %s'." program-name cmds-help.human)

   :program-version
   (string.format "AGPLv3 -- %s" program-version)

   :file-already-exists
   (string.format "A file or directory named '%s' already exists." program-file)

   :date-already-exists
   (.. "The following event already exists in your agenda:" newline
       "%s %s" newline
       "Choose one of the options below:" newline
       "1. Replace the old event" newline
       "2. Add the event to the existing date" newline
       "3. Cancell even creation" newline
       "> ")

   :file-doesnt-exist
   (string.format "%s doesn't exist. Try running '%s %s' (without the quotation marks)."
                  program-file
                  program-name
                  cmd-init)

   :cancelling-date-creation
   "Cancelling date creation"

   :not-an-option
   "Error: '%s' is not an option."

   :date-format
   (.. "Error: Date format must be mmdd" newline
       "Example: 1231 \"The last day of december\"")

   :key-doesnt-exist
   "The date '%s' wasn't found in your agenda."

   :sub-key-doesnt-exist
   "The item '%s' on %s wasn't found in your agenda."

   :agenda-empty
   "There is nothing in your agenda."

   :initialized
   (string.format "Successfully created %s" program-file)

   :added
   "Added '%s' to your agenda."

   :removed
   "Removed '%s' from your agenda."})

(fn key-exists? [tbl key]
  (if (= (type tbl) :table)
    (let [tbl-keys (lume.keys tbl)]
      (if (lume.any tbl-keys #(= key $1))
        true
        false))
    false))

(fn get-date []
  (os.date "%m%d"))

;; Not in use (yet?)
;; (fn get-day []
;;   (string.lower (os.date day-fmt)))

(fn file-exists? [str]
  (let [file-in (io.open str :r)]
    (if file-in
      (file-in:close)
      false)))

(fn file->table [str]
  (if (file-exists? str)
    (with-open [file-in (io.open str :r)]
               (lume.deserialize (file-in:read :*all)))
    nil))

(fn table->file [str tbl]
  (if (file-exists? str)
    (with-open [file-out (io.open str :w)]
               (file-out:write (lume.serialize tbl)))
    nil))

(fn create-prefix [seq key]
  (if (> (# seq) 1)
    (string.format "%s. " key)
    ""))

(fn print-format [str ...]
  (print (string.format str ...)))

(fn init/create-file []
  (with-open [file-out (io.open program-file :w)]
             (file-out:write (lume.serialize []))
             (print messages.initialized)))

(fn init []
  (if (file-exists? program-file)
    (print messages.file-already-exists)
    (init/create-file)))

(fn add/add-event [date event-str]
  (let [tbl (file->table program-file)]
    (if (key-exists? tbl date)
      (tset tbl date (+ 1 (length (. tbl date))) event-str)
      (tset tbl date [event-str]))
    (table->file program-file tbl)
    (print-format messages.added event-str)))

(fn add [date-str event-str]
  (if (file-exists? program-file)
    (let [date (if (= date-str keyword-today) (get-date) date-str)]
      (add/add-event date event-str))
    (print messages.file-doesnt-exist)))

(fn rm [date-str event-str]
  (if (file-exists? program-file)
    (let [tbl              (file->table program-file)
          date             (if (= date-str keyword-today) (get-date) date-str)
          events-seq       (. tbl date)
          events-seq-index (tonumber event-str)
          date-exists?     (key-exists? tbl date)
          event-exists?    (key-exists? events-seq events-seq-index)]
      (match [date-exists? event-exists? event-str]
        ;; If only the date is given
        [true false nil]
        (do (tset tbl date nil)
          (table->file program-file tbl)
          (print-format messages.removed date))

        ;; If date and event exist
        [true true]
        (let [event (. events-seq events-seq-index)]
          (table.remove events-seq events-seq-index)
          (if (= (# events-seq) 0)
            (do (tset tbl date nil)
              (table->file program-file tbl)
              (print-format messages.removed event))
            (do (tset tbl date events-seq)
              (table->file program-file tbl)
              (print-format messages.removed event))))

        ;; If date exists, but event doesn't
        [true false]
        (print-format messages.sub-key-doesnt-exist event-str date)

        ;; If date doesn't exist
        [false]
        (print-format messages.key-doesnt-exist date)))
    (print messages.file-doesnt-exist)))

(fn notify []
  (if (file-exists? program-file)
    (when (key-exists? (file->table program-file) (get-date))
      (io.write event-symbol))
    (print messages.file-doesnt-exist)))

(fn ls/print-seq [events-seq]
  (let [keys (lume.keys events-seq)]
    ;; Keys are used here instead of the sequence values, so the keys
    ;; print out in order
    (each [_ key (pairs keys)]
      (let [event        (. events-seq key)
            event-prefix (create-prefix events-seq key)]
        (print-format "%s%s" event-prefix event)))))

(fn ls/ls-date [date-str]
  (if (file-exists? program-file)
    (let [tbl  (file->table program-file)
          date (if (= date-str keyword-today) (get-date) date-str)]
      (when (key-exists? tbl date)
        (let [events-seq (. tbl date)]
          (ls/print-seq events-seq))))
    (print messages.file-doesnt-exist)))

(fn ls/print-seq [events-seq date]
  (let [[first-event]           events-seq
        [first-key & rest-keys] (lume.keys events-seq)
        first-event-prefix      (create-prefix events-seq first-key)]
    (print-format "%s%s" first-event-prefix first-event)
    ;; The rest-keys is used here instead of a rest-events, so the
    ;; keys for the underlying table in the sequence don't restart at 1
    (each [_ key (pairs rest-keys)]
      (let [event (. events-seq key)
            padding (string.rep " " (+ 1 (# date)))]
        (print-format "%s %s. %s" padding key event)))))

(fn ls/sort-and-print-tbl [tbl tbl-keys]
  (table.sort tbl-keys)
  (each [_ date (pairs tbl-keys)]
    ;; Print the date without trailing newline character
    (io.write (string.format "%s: " date))
    (let [events-seq (. tbl date)]
      (ls/print-seq events-seq date))))

(fn ls [date-str]
  (if (file-exists? program-file)
    (if date-str
      (ls/ls-date date-str)
      (let [tbl        (file->table program-file)
            tbl-keys   (lume.keys tbl)
            tbl-length (# tbl-keys)]
        (if (> tbl-length 0)
          (ls/sort-and-print-tbl tbl tbl-keys)
          (print messages.agenda-empty))))
    (print messages.file-doesnt-exist)))

(fn help []
  (print
    (.. "Usage:\n"
        "  fa <command> [<arg>] [<arg>]\n"
        "\n"
        "Commands:\n"
        "  init - Creates your agenda.\n"
        "  add <mmdd|today> \"Quoted text\" - Adds an event to your agenda.\n"
        "  rm <mmdd|today> - Removes a given date from your agenda.\n"
        "  rm <mmdd|today> [<number>] - Removes an event from given date from your agenda.\n"
        "  ls [<mmdd|today>] - Lists the events on the given date.\n"
        "  ls - Lists all dates and their events.\n"
        "  notify - Displays a \"!\" if an event exists today.\n"
        "  version - Prints the current version of fa.\n"
        "\n"
        "Examples:\n"
        "  fa init\n"
        "  fa add 1231 \"Sherry's birthday\"\n"
        "  fa add today \"Sherry's birthday\"\n"
        "  fa rm 1231\n"
        "  fa rm 1231 3 (See note below)\n"
        "  fa rm today\n"
        "  fa rm today 3 (See note below)\n"
        "  fa ls today\n"
        "  fa ls 1231\n"
        "  fa ls\n"
        "  fa notify\n"
        "  fa version\n"
        "\n"
        "Note: You may need to run 'fa ls' to see which number correlates to which event.\n")))

(fn process-args [arg-tbl]
  (match arg-tbl
    [cmd-add date-str event-str nil]
    (add date-str event-str)

    [cmd-rm date-str event-str nil]
    (rm date-str event-str)

    [cmd-rm date-str nil]
    (rm date-str)

    [cmd-ls date-str nil]
    (ls date-str)

    [cmd-ls nil]
    (ls)

    ([cmd nil] ? (or (= cmd cmds-help.human)
                     (= cmd cmds-help.long)
                     (= cmd cmds-help.short)))
    (help)

    ([cmd nil] ?  (or (= cmd cmds-version.human)
                      (= cmd cmds-version.long)
                      (= cmd cmds-version.short)))
    (print messages.program-version)

    [cmd-init nil]
    (init)

    [cmd-notify nil]
    (notify)

    _ (print messages.usage)))

(fn main [arg-tbl]
  (process-args arg-tbl))

(main arg)
